{ DADE Plug - In for SQLDirect by Yuri Sheino
  Written by Keith Price for Centrelink
  7/2001 }

unit daSQLDirect;

interface

{$I ppIfDef.pas}

uses Classes, SysUtils, Forms, ExtCtrls, DB, Contnrs,
  ppClass, ppComm, ppDBPipe, ppDB, ppClasUt, ppTypes,
  daDB, daQueryDataView, daDataView, daPreviewDataDlg,
  SDEngine;

type

  {SQLDirect DataView Classes:

     1.  SD TDataSet descendants
           - TDataSets that can be children of a DataView.
           - Override the HasParent method of TComponent to return True
           - Must be registered with the Delphi IDE using the RegisterNoIcon procedure

       a. TdaChildSDQuery - TSDQuery descendant that can be a child of a DataView
       b. TdaChildSDStoredProc - TSDStoredProc descendant that can be a child of a DataView

     2.  TdaSDSession
           - descendant of TppSession
           - implements GetDatabaseNames, GetTableNames, etc.

     3.  TdaSDDataSet
          - descendant of TppDataSet
          - implements GetFieldNames for SQL

     4.  TdaSDQueryDataView
          - descendant of TppQueryDataView
          - uses the above classes to create the required
            Query -> DataSource -> Pipeline -> Report connection
          - uses the TdaSQL object built by the QueryWizard to assign
            SQL to the TSDQuery etc.

      }

  { TdaChildSDQuery }
  TdaChildSDQuery = class(TSDQuery)
  public
    function HasParent: Boolean; override;
  end; {class, TdaChildSDQuery}

  { TdaChildSDStoredProc }
  TdaChildSDStoredProc = class(TSDStoredProc)
  public
    function HasParent: Boolean; override;
  end; {class, TdaChildSDStoredProc}

  { TdaSDSession }
  TdaSDSession = class(TdaSession)
  private

    procedure AddDatabase(aDatabase: TComponent);

  protected

    procedure SetDataOwner(aDataOwner: TComponent); override;

  public

    class function ClassDescription: string; override;
    class function DataSetClass: TdaDataSetClass; override;
    class function DatabaseClass: TComponentClass; override;

    procedure GetDatabaseNames(aList: TStrings); override;
    function GetDatabaseType(const aDatabaseName: string): TppDatabaseType; override;
    procedure GetTableNames(const aDatabaseName: string; aList: TStrings); override;
    function ValidDatabaseTypes: TppDatabaseTypes; override;

  end; {class, TdaSDSession}

  { TdaSDDataSet }
  TdaSDDataSet = class(TdaDataSet)
  private
    FQuery: TSDQuery;

    function GetQuery: TSDQuery;

  protected
    procedure BuildFieldList; override;
    function GetActive: Boolean; override;
    procedure SetActive(Value: Boolean); override;
    procedure SetDatabaseName(const aDatabaseName: string); override;
    procedure SetDataName(const aDataName: string); override;

    property Query: TSDQuery read GetQuery;

  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    class function ClassDescription: string; override;

    procedure GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings); override;
    procedure GetFieldsForSQL(aList: TList; aSQL: TStrings); override;

  end; {class, TdaSDDataSet}


  { TdaSDQueryDataView }
  TdaSDQueryDataView = class(TdaQueryDataView)
  private
    FDataSource: TppChildDataSource;
    FQuery: TdaChildSDQuery;

  protected
    procedure SQLChanged; override;

  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    class function PreviewFormClass: TFormClass; override;
    class function SessionClass: TClass; override;

    procedure Init; override;
    procedure ConnectPipelinesToData; override;

  published
    property DataSource: TppChildDataSource read FDataSource;

  end; {class, TdaSDQueryDataView}


  {global functions to access default SD session and database}
function daGetDefaultSDDatabase: TSDDatabase;

  {utility routines}
procedure daGetSDDatabaseNames(aList: TStrings);
function daGetSDDatabaseForName(aDatabaseName: string): TSDDatabase;

function daGetSDDatabaseList: TppComponentList;

  {Delphi design time registration}
procedure Register;


implementation

const
  cDefaultDatabase = 'D10ADV01';
  cDefaultDatabaseName = 'ITMSDB2DB';
var
  FSDDatabase: TSDDatabase;
  FSDDatabaseList: TppComponentList;


{******************************************************************************
 *
 ** R E G I S T E R
 *
{******************************************************************************}

procedure Register;
begin

  {SD DataAccess Components}
  RegisterNoIcon([TdaChildSDQuery, TdaChildSDStoredProc]);

  {SD DataViews}
  RegisterNoIcon([TdaSDQueryDataView]);

end;


{******************************************************************************
 *
 ** C H I L D   S D  D A T A   A C C E S S   C O M P O N  E N T S
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaChildSDQuery.HasParent }

function TdaChildSDQuery.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{------------------------------------------------------------------------------}
{ TdaChildSDStoredProc.HasParent }

function TdaChildSDStoredProc.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{******************************************************************************
 *
 ** S D   S E S S I O N
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaSDSession.ClassDescription }

class function TdaSDSession.ClassDescription: string;
begin
  Result := 'SDSession';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaSDSession.DataSetClass }

class function TdaSDSession.DataSetClass: TdaDataSetClass;
begin
  Result := TdaSDDataSet;
end; {class function, DataSetClass}

{------------------------------------------------------------------------------}
{ TdaSDSession.DatabaseClass }

class function TdaSDSession.DatabaseClass: TComponentClass;
begin
  Result := TSDDatabase;
end;

{------------------------------------------------------------------------------}
{ TdaSDSession.GetTableNames }

procedure TdaSDSession.GetTableNames(const aDatabaseName: string; aList: TStrings);
var
  lSession: TSDSession;
  lDatabase: TSDDatabase;

begin
  {get the database}
  lDatabase := daGetSDDatabaseForName(aDatabaseName);
  {get the session}
  lSession := lDatabase.Session;

  {connection must be active to get table names}
  if not lDatabase.Connected then
    lDatabase.Connected := True;

  {get list of table names from the session object}
  if lDatabase.Connected then
    lSession.GetTableNames(lDatabase.DatabaseName, '', false, aList);

end; {procedure, GetTableNames}

{------------------------------------------------------------------------------}
{ TdaSDSession.AddDatabase }

procedure TdaSDSession.AddDatabase(aDatabase: TComponent);
begin

  if daGetSDDatabaseList.IndexOf(aDatabase) < 0 then
    FSDDatabaseList.Add(aDatabase);

end; {procedure, AddDatabase}

{------------------------------------------------------------------------------}
{ TdaSDSession.GetDatabaseNames }

procedure TdaSDSession.GetDatabaseNames(aList: TStrings);
var
  liIndex: Integer;

begin
  {call utility routine to get list of database names}
  daGetSDDatabaseNames(aList);

  daGetDatabaseObjectsFromOwner(TdaSessionClass(Self.ClassType), aList, DataOwner);

  for liIndex := 0 to aList.Count - 1 do
    if aList.Objects[liIndex] <> nil then
      AddDatabase(TComponent(aList.Objects[liIndex]));

end; {procedure, GetDatabaseNames}

{------------------------------------------------------------------------------}
{ TdaSDSession.SetDataOwner }

procedure TdaSDSession.SetDataOwner(aDataOwner: TComponent);
var
  lList: TStringList;
begin

  inherited SetDataOwner(aDataOwner);

  lList := TStringList.Create;

  GetDatabaseNames(lList);

  lList.Free;

end; {procedure, SetDataOwner}

{------------------------------------------------------------------------------}
{ TdaSDSession.ValidDatabaseTypes }

function TdaSDSession.ValidDatabaseTypes: TppDatabaseTypes;
begin // Only some database types supported, viz. DB2, Oracle, MS SQLServer
        // Sybase ASE (see below about ASA), MySQL and Interbase - Sybase ASA,
        // Centura SQLBase, Informix and ODBC datasources supported by
        // SQLDirect but not this plug-in.
        // To add support, ReportBuilder source must be further modified
        // (and SQLS.
  Result := [dtDB2, dtOracle, dtMSSQLServer, dtSybaseASE, dtInterBase, dtMySQL];
end; {procedure, ValidDatabaseTypes}

{------------------------------------------------------------------------------}
{ TdaSDSession.GetDatabaseType }

function TdaSDSession.GetDatabaseType(const aDatabaseName: string): TppDatabaseType;
var
  lDatabase: TSDDatabase;
begin
  lDatabase := daGetSDDatabaseForName(aDatabaseName);
  case lDatabase.ServerType of
    stDB2: Result := dtDB2;
    stOracle: Result := dtOracle;
    stSQLServer: Result := dtMSSQLServer;
    stInterbase: Result := dtInterBase;
    stMySQL: Result := dtMySQL;
    stSybase: Result := dtSybaseASE; // SQLDirect has a common code base for Sybase ASE/ASA
  else
    Result := dtDB2;
  end;
end; {function, GetDatabaseType}


{******************************************************************************
 *
 ** S D  D A T A S E T
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.Create }

constructor TdaSDDataSet.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  FQuery := nil;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.Destroy }

destructor TdaSDDataSet.Destroy;
begin

  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.ClassDescription }

class function TdaSDDataSet.ClassDescription: string;
begin
  Result := 'SDDataSet';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.GetActive }

function TdaSDDataSet.GetActive: Boolean;
begin
  Result := GetQuery.Active
end; {function, GetActive}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.SetActive }

procedure TdaSDDataSet.SetActive(Value: Boolean);
begin
  GetQuery.Active := Value;
end; {procedure, SetActive}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.GetQuery }

function TdaSDDataSet.GetQuery: TSDQuery;
begin

  {create SD table, if needed}
  if (FQuery = nil) then
    FQuery := TSDQuery.Create(Self);

  Result := FQuery;

end; {procedure, GetQuery}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.SetDatabaseName }

procedure TdaSDDataSet.SetDatabaseName(const aDatabaseName: string);
var lDatabase: TSDDatabase;
  lSession: TSDSession;
begin
  {get the database}
  lDatabase := daGetSDDatabaseForName(aDatabaseName);
  {get the session}
  lSession := lDatabase.Session;
  // aDatebaseName is the TSDDatabase COMPONENT name: must set DatabaseName
  // to the TSDDatabase.DatabaseName instead.
  inherited SetDatabaseName(lDatabase.DatabaseName);

  {table cannot be active to set database property}
  if GetQuery.Active then
    FQuery.Active := False;

  {Set SD database and sesion name}
  FQuery.SessionName := lSession.SessionName;
  FQuery.DatabaseName := lDatabase.DatabaseName;
end; {procedure, SetDatabaseName}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.SetDataName }

procedure TdaSDDataSet.SetDataName(const aDataName: string);
begin

  inherited SetDataName(aDataName);

  {dataset cannot be active to set data name}
  if GetQuery.Active then
    FQuery.Active := False;

  {construct an SQL statment that returns an empty result set,
   this is used to get the field information }
  FQuery.SQL.Text := 'SELECT * FROM ' + aDataName +
    ' WHERE ''c'' <> ''c'' ';

end; {procedure, SetDataName}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.BuildFieldList }

procedure TdaSDDataSet.BuildFieldList;
var
  liIndex: Integer;
  lQueryField: TField;
  lField: TppField;
begin

  inherited BuildFieldList;

  {set table to active}
  if not (GetQuery.Active) then
    FQuery.Active := True;

  {create TppField objects for each field in the table}
  for liIndex := 0 to FQuery.FieldCount - 1 do
  begin
    lQueryField := FQuery.Fields[liIndex];

    lField := TppField.Create(nil);

    lField.TableName := DataName;
    lField.FieldName := lQueryField.FieldName;
    lField.FieldAlias := lQueryField.DisplayLabel;
    lField.FieldLength := lQueryField.Size;
    lField.DataType := ppConvertFieldType(lQueryField.DataType);
    lField.DisplayWidth := lQueryField.DisplayWidth;

    AddField(lField);
  end;

end; {function, BuildFieldList}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.GetFieldNamesForSQL }

procedure TdaSDDataSet.GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings);
var
  lQuery: TSDQuery;
  lDatabase: TSDDatabase;
begin
  aList.Clear;
  {get the database}
  lDatabase := daGetSDDatabaseForName(DatabaseName);
  {create a temporary SD query}
  lQuery := TSDQuery.Create(Self);
  try
    {set the database and SQL properties}
    lQuery.DatabaseName := DatabaseName;
    lQuery.SessionName := lDatabase.Session.SessionName;
    lQuery.SQL := aSQL;

    {get the field names}
    lQuery.GetFieldNames(aList);
  finally
    lQuery.Free;
  end;
end; {procedure, GetFieldNamesForSQL}

{------------------------------------------------------------------------------}
{ TdaSDDataSet.GetFieldsForSQL }

procedure TdaSDDataSet.GetFieldsForSQL(aList: TList; aSQL: TStrings);
var
  lQuery: TSDQuery;
  lDatabase: TSDDatabase;
  lQueryField: TField;
  lField: TppField;
  liIndex: Integer;
begin
  aList.Clear;

  {create a temporary SD query}
  lQuery := TSDQuery.Create(Self);

  try
    {assign database and SQL properties}
    {get the database}
    lDatabase := daGetSDDatabaseForName(DatabaseName);
    lQuery.SessionName := lDatabase.Session.SessionName;
    lQuery.DatabaseName := DatabaseName;
    lQuery.SQL := aSQL;

    {set query to active}
    lQuery.Active := True;

    {create a TppField object for each field in the query}
    for liIndex := 0 to lQuery.FieldCount - 1 do
    begin
      lQueryField := lQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.FieldName := lQueryField.FieldName;
      lField.FieldAlias := lQueryField.DisplayLabel;
      lField.FieldLength := lQueryField.Size;
      lField.DataType := ppConvertFieldType(lQueryField.DataType);
      lField.DisplayWidth := lQueryField.DisplayWidth;

      aList.Add(lField);
    end;

  finally
    lQuery.Free;

  end;

end; {procedure, GetFieldsForSQL}




{******************************************************************************
 *
 ** S D  Q U E R Y   D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaSDQueryDataView.Create }

constructor TdaSDQueryDataView.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  {notes: 1. must use ChildQuery, ChildDataSource, ChildPipeline etc.
          2. use Self as owner for Query, DataSource etc.
          3. do NOT assign a Name }

  FQuery := TdaChildSDQuery.Create(Self);

  FDataSource := TppChildDataSource.Create(Self);
  FDataSource.DataSet := FQuery;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaSDQueryDataView.Destroy }

destructor TdaSDQueryDataView.Destroy;
begin
  FDataSource.Free;
  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaSDQueryDataView.SessionClass }

class function TdaSDQueryDataView.SessionClass: TClass;
begin
  Result := TdaSDSession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdaSDQueryDataView.PreviewFormClass }

class function TdaSDQueryDataView.PreviewFormClass: TFormClass;
begin
  Result := TFormClass(GetClass('TdaPreviewDataDialog'));
end; {class function, PreviewFormClass}


{------------------------------------------------------------------------------}
{ TdaSDQueryDataView.ConnectPipelinesToData }

procedure TdaSDQueryDataView.ConnectPipelinesToData;
begin

  if DataPipelineCount = 0 then Exit;

  {need to reconnect here}
  TppDBPipeline(DataPipelines[0]).DataSource := FDataSource;


end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdaSDQueryDataView.Init }

procedure TdaSDQueryDataView.Init;
var
  lDataPipeline: TppChildDBPipeline;

begin

  inherited Init;

  if DataPipelineCount > 0 then Exit;

  {note: DataView's owner must own the DataPipeline }
  lDataPipeline := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));
  lDataPipeline.DataSource := FDataSource;

  lDataPipeline.AutoCreateFields := False;

  {add DataPipeline to the dataview }
  lDataPipeline.DataView := Self;

end; {procedure, Init}

{------------------------------------------------------------------------------}
{ TdaSDQueryDataView.SQLChanged }

procedure TdaSDQueryDataView.SQLChanged;
var lDatabase: TSDDatabase;
begin

  if FQuery.Active then
    FQuery.Close;
  lDatabase := daGetSDDatabaseForName(SQL.DatabaseName);
  FQuery.SessionName := lDatabase.SessionName;
  FQuery.DatabaseName := lDatabase.DatabaseName;
  FQuery.SQL := SQL.MagicSQLText;

end; {procedure, SQLChanged}

{******************************************************************************
 *
 ** P R O C E D U R E S   A N D   F U N C T I O N S
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ daGetDefaultSDDatabase }

function daGetDefaultSDDatabase: TSDDatabase;
begin

  {create the default SD database, if needed}
  if (FSDDatabase = nil) then
  begin
      {create default SD database}
    FSDDatabase := TSDDatabase.Create(nil);
    FSDDatabase.DatabaseName := cDefaultDatabase;
    FSDDatabase.RemoteDatabase := cDefaultDatabase;
    FSDDatabase.Name := cDefaultDatabaseName;
  end;

  Result := FSDDatabase;

end; {function, daGetDefaultSDDatabase}

{------------------------------------------------------------------------------}
{ daGetSDDatabaseNames }

procedure daGetSDDatabaseNames(aList: TStrings);
begin

  {could add hard-coded connection strings here, or could
   read from an .ini file }

end; {procedure, daGetSDDatabaseNames}

{------------------------------------------------------------------------------}
{ daGetSDDatabaseForName }

function daGetSDDatabaseForName(aDatabaseName: string): TSDDatabase;
var
  liIndex: Integer;

begin
  Result := nil;

  liIndex := 0;

 {check for a database object with this name}
  while (Result = nil) and (liIndex < daGetSDDatabaseList.Count) do
  begin
    if (AnsiCompareText(FSDDatabaseList[liIndex].Name, aDatabaseName) = 0) or
      (AnsiCompareText(TSDDatabase(FSDDatabaseList[liIndex]).DatabaseName, aDatabaseName) = 0) then
      Result := TSDDatabase(FSDDatabaseList[liIndex]);
    Inc(liIndex);
  end;

  if (Result <> nil) then Exit;


  {use the default database object}
  Result := daGetDefaultSDDatabase;

  {set DatabaseName property, if needed}
  if (Result.DatabaseName <> aDatabaseName) then
  begin

    if Result.Connected then
      Result.Connected := False;

    Result.DatabaseName := aDatabaseName;

  end;

end; {function, daGetSDDatabaseForName}

{------------------------------------------------------------------------------}
{ daGetSDDatabaseList }

function daGetSDDatabaseList: TppComponentList;
begin
  if (FSDDatabaseList = nil) then
    FSDDatabaseList := TppComponentList.Create(nil);

  Result := FSDDatabaseList;

end; {function, daGetSDDatabaseList}


initialization

  {register the SD descendant classes}
  RegisterClasses([TdaChildSDQuery, TdaChildSDStoredProc]);

  {register DADE descendant session, dataset, dataview}
  daRegisterSession(TdaSDSession);
  daRegisterDataSet(TdaSDDataSet);
  daRegisterDataView(TdaSDQueryDataView);

  {initialize internal reference variables}
  FSDDatabase := nil;
  FSDDatabaseList := nil;


finalization

  {free the default database object}
  FSDDatabase.Free;

  FSDDatabaseList.Free;

  {unregister the SD descendant classes}
  UnRegisterClasses([TdaChildSDQuery, TdaChildSDStoredProc]);

  {unregister DADE descendant the session, dataset, dataview}
  daUnRegisterSession(TdaSDSession);
  daUnRegisterDataSet(TdaSDDataSet);
  daUnRegisterDataView(TdaSDQueryDataView);


end.

